@@ -1,10 +1,16 @@
+/* $Id: CoinHelperFunctions.hpp 1666 2013-11-25 05:04:09Z tkr $ */
 // Copyright (C) 2000, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
 #ifndef CoinHelperFunctions_H
 #define CoinHelperFunctions_H
+
+#include "CoinUtilsConfig.h"
+
 #if defined(_MSC_VER)
 #  include <direct.h>
+#  include <cctype>
 #  define getcwd _getcwd
 #else
 #  include <unistd.h>
@@ -13,8 +19,19 @@
 
 #include <cstdlib>
 #include <cstdio>
+#include <algorithm>
+#include "CoinTypes.hpp"
 #include "CoinError.hpp"
-#include "CoinFinite.hpp"
+
+// Compilers can produce better code if they know about __restrict
+#ifndef COIN_RESTRICT
+#ifdef COIN_USE_RESTRICT
+#define COIN_RESTRICT __restrict
+#else
+#define COIN_RESTRICT
+#endif
+#endif
+
 //#############################################################################
 
 /** This helper function copies an array to another location using Duff's
@@ -28,9 +45,11 @@ CoinCopyN(register const T* from, const int size, register T* to)
     if (size == 0 || from == to)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to copy negative number of entries",
 			"CoinCopyN", "");
+#endif
 
     register int n = (size + 7) / 8;
     if (to > from) {
@@ -71,11 +90,17 @@ CoinCopyN(register const T* from, const int size, register T* to)
 /** This helper function copies an array to another location using Duff's
     device (for a speedup of ~2). The source array is given by its first and
     "after last" entry; the target array is given by its first entry.
-    Overlapping arrays are handled correctly. */
+    Overlapping arrays are handled correctly.
+
+    All of the various CoinCopyN variants use an int for size. On 64-bit
+    architectures, the address diff last-first will be a 64-bit quantity.
+    Given that everything else uses an int, I'm going to choose to kick
+    the difference down to int.  -- lh, 100823 --
+*/
 template <class T> inline void
 CoinCopy(register const T* first, register const T* last, register T* to)
 {
-    CoinCopyN(first, last - first, to);
+    CoinCopyN(first, static_cast<int>(last-first), to);
 }
 
 //-----------------------------------------------------------------------------
@@ -94,9 +119,11 @@ CoinDisjointCopyN(register const T* from, const int size, register T* to)
     if (size == 0 || from == to)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to copy negative number of entries",
 			"CoinDisjointCopyN", "");
+#endif
 
 #if 0
     /* There is no point to do this test. If to and from are from different
@@ -253,9 +280,11 @@ CoinMemcpyN(register const T* from, const int size, register T* to)
     if (size == 0 || from == to)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to copy negative number of entries",
 			"CoinMemcpyN", "");
+#endif
 
 #if 0
     /* There is no point to do this test. If to and from are from different
@@ -333,10 +362,11 @@ CoinFillN(register T* to, const int size, register const T value)
     if (size == 0)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to fill negative number of entries",
 			"CoinFillN", "");
-
+#endif
 #if 1
     for (register int n = size / 8; n > 0; --n, to += 8) {
 	to[0] = value;
@@ -411,9 +441,11 @@ CoinZeroN(register T* to, const int size)
     if (size == 0)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to fill negative number of entries",
 			"CoinZeroN", "");
+#endif
 #if 1
     for (register int n = size / 8; n > 0; --n, to += 8) {
 	to[0] = 0;
@@ -550,9 +582,10 @@ CoinIsSorted(register const T* first, const int size)
     if (size == 0)
 	return true;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("negative number of entries", "CoinIsSorted", "");
-
+#endif
 #if 1
     // size1 is the number of comparisons to be made
     const int size1 = size  - 1;
@@ -609,9 +642,10 @@ CoinIotaN(register T* first, const int size, register T init)
     if (size == 0)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("negative number of entries", "CoinIotaN", "");
-
+#endif
 #if 1
     for (register int n = size / 8; n > 0; --n, first += 8, init += 8) {
 	first[0] = init;
@@ -674,7 +708,7 @@ template <class T> inline T *
 CoinDeleteEntriesFromArray(register T * arrayFirst, register T * arrayLast,
 			   const int * firstDelPos, const int * lastDelPos)
 {
-    int delNum = lastDelPos - firstDelPos;
+    int delNum = static_cast<int>(lastDelPos - firstDelPos);
     if (delNum == 0)
 	return arrayLast;
 
@@ -689,7 +723,8 @@ CoinDeleteEntriesFromArray(register T * arrayFirst, register T * arrayLast,
 	delSortedPos = new int[delNum];
 	CoinDisjointCopy(firstDelPos, lastDelPos, delSortedPos);
 	std::sort(delSortedPos, delSortedPos + delNum);
-	delNum = std::unique(delSortedPos, delSortedPos + delNum) - delSortedPos;
+	delNum = static_cast<int>(std::unique(delSortedPos,
+				  delSortedPos+delNum) - delSortedPos);
     }
     const int * delSorted = delSortedPos ? delSortedPos : firstDelPos;
 
@@ -703,7 +738,7 @@ CoinDeleteEntriesFromArray(register T * arrayFirst, register T * arrayLast,
 	size += copyLast - copyFirst;
     }
     const int copyFirst = delSorted[last] + 1;
-    const int copyLast = arrayLast - arrayFirst;
+    const int copyLast = static_cast<int>(arrayLast - arrayFirst);
     CoinCopy(arrayFirst + copyFirst, arrayFirst + copyLast,
 	     arrayFirst + size);
     size += copyLast - copyFirst;
@@ -722,12 +757,19 @@ CoinDeleteEntriesFromArray(register T * arrayFirst, register T * arrayLast,
 /* Thanks to Stefano Gliozzi for providing an operating system
    independent random number generator.  */
 
-// linear congruential generator. given the seed, the generated numbers are  
-// always the same regardless the (32 bit) architecture. This allows to 
-// build & test in different environments (i.e. Wintel, Linux/Intel AIX Power5)
-// getting in most cases the same optimization path. 
-/// Return random number between 0 and 1.
-inline double CoinDrand48(bool isSeed = false, unsigned int seed=1)
+/*! \brief Return a random number between 0 and 1
+
+  A platform-independent linear congruential generator. For a given seed, the
+  generated sequence is always the same regardless of the (32-bit)
+  architecture. This allows to build & test in different environments, getting
+  in most cases the same optimization path.
+
+  Set \p isSeed to true and supply an integer seed to set the seed
+  (vid. #CoinSeedRandom)
+
+  \todo Anyone want to volunteer an upgrade for 64-bit architectures?
+*/
+inline double CoinDrand48 (bool isSeed = false, unsigned int seed = 1)
 {
   static unsigned int last = 123456;
   if (isSeed) { 
@@ -736,9 +778,10 @@ inline double CoinDrand48(bool isSeed = false, unsigned int seed=1)
     last = 1664525*last+1013904223;
     return ((static_cast<double> (last))/4294967296.0);
   }
-  return(0.0);
+  return (0.0);
 }
-/// Seed random number generator
+
+/// Set the seed for the random number generator
 inline void CoinSeedRandom(int iseed)
 {
   CoinDrand48(true, iseed);
@@ -748,12 +791,16 @@ inline void CoinSeedRandom(int iseed)
 
 #if defined(_MSC_VER) || defined(__MINGW32__) || defined(__CYGWIN32__)
 
+/// Return a random number between 0 and 1
 inline double CoinDrand48() { return rand() / (double) RAND_MAX; }
+/// Set the seed for the random number generator
 inline void CoinSeedRandom(int iseed) { srand(iseed + 69822); }
 
 #else
 
+/// Return a random number between 0 and 1
 inline double CoinDrand48() { return drand48(); }
+/// Set the seed for the random number generator
 inline void CoinSeedRandom(int iseed) { srand48(iseed + 69822); }
 
 #endif
@@ -794,8 +841,8 @@ inline int CoinStrNCaseCmp(const char* s0, const char* s1,
 	if (s1[i] == 0) {
 	    return 1;
 	}
-	const int c0 = tolower(s0[i]);
-	const int c1 = tolower(s1[i]);
+	const int c0 = std::tolower(s0[i]);
+	const int c1 = std::tolower(s1[i]);
 	if (c0 < c1)
 	    return -1;
 	if (c0 > c1)
@@ -821,19 +868,22 @@ template <class T> inline void CoinSwap (T &x, T &y)
 */
 
 template <class T> inline int
-CoinToFile( const T* array, int size, FILE * fp)
+CoinToFile( const T* array, CoinBigIndex size, FILE * fp)
 {
-    int numberWritten;
+    CoinBigIndex numberWritten;
     if (array&&size) {
-	numberWritten = fwrite(&size,sizeof(int),1,fp);
+	numberWritten =
+	    static_cast<CoinBigIndex>(fwrite(&size,sizeof(int),1,fp));
 	if (numberWritten!=1)
 	    return 1;
-	numberWritten = fwrite(array,sizeof(T),size,fp);
+	numberWritten =
+	    static_cast<CoinBigIndex>(fwrite(array,sizeof(T),size_t(size),fp));
 	if (numberWritten!=size)
 	    return 1;
     } else {
 	size = 0;
-	numberWritten = fwrite(&size,sizeof(int),1,fp);
+	numberWritten = 
+	    static_cast<CoinBigIndex>(fwrite(&size,sizeof(int),1,fp));
 	if (numberWritten!=1)
 	    return 1;
     }
@@ -849,10 +899,11 @@ CoinToFile( const T* array, int size, FILE * fp)
 */
 
 template <class T> inline int
-CoinFromFile( T* &array, int size, FILE * fp,int & newSize)
+CoinFromFile( T* &array, CoinBigIndex size, FILE * fp, CoinBigIndex & newSize)
 {
-    int numberRead;
-    numberRead = fread(&newSize,sizeof(int),1,fp);
+    CoinBigIndex numberRead;
+    numberRead =
+        static_cast<CoinBigIndex>(fread(&newSize,sizeof(int),1,fp));
     if (numberRead!=1)
 	return 1;
     int returnCode=0;
@@ -860,7 +911,8 @@ CoinFromFile( T* &array, int size, FILE * fp,int & newSize)
 	returnCode=2;
     if (newSize) {
 	array = new T [newSize];
-	numberRead = fread(array,sizeof(T),newSize,fp);
+	numberRead =
+	    static_cast<CoinBigIndex>(fread(array,sizeof(T),newSize,fp));
 	if (numberRead!=newSize)
 	    returnCode=1;
     } else {
@@ -872,6 +924,7 @@ CoinFromFile( T* &array, int size, FILE * fp,int & newSize)
 //#############################################################################
 
 /// Cube Root
+#if 0
 inline double CoinCbrt(double x)
 {
 #if defined(_MSC_VER) 
@@ -880,6 +933,19 @@ inline double CoinCbrt(double x)
     return cbrt(x);
 #endif
 }
+#endif
+
+//-----------------------------------------------------------------------------
+
+/// This helper returns "sizeof" as an int 
+#define CoinSizeofAsInt(type) (static_cast<int>(sizeof(type)))
+/// This helper returns "strlen" as an int 
+inline int
+CoinStrlenAsInt(const char * string)
+{
+    return static_cast<int>(strlen(string));
+}
+
 /** Class for thread specific random numbers
 */
 #if defined COIN_OWN_RANDOM_32
@@ -920,6 +986,11 @@ public:
   { 
     seed_ = seed;
   }
+  /** Get seed. */
+  inline unsigned int getSeed() const
+  { 
+    return seed_;
+  }
   /// return a random number
   inline double randomDouble() const
   {
@@ -928,6 +999,14 @@ public:
     retVal = ((static_cast<double> (seed_))/4294967296.0);
     return retVal;
   }
+  /// make more random (i.e. for startup)
+  inline void randomize(int n=0)
+  {
+    if (!n) 
+      n=seed_ & 255;
+    for (int i=0;i<n;i++)
+      randomDouble();
+  }
   //@}
   
   
@@ -1001,6 +1080,16 @@ public:
 #endif
     return retVal;
   }
+  /// make more random (i.e. for startup)
+  inline void randomize(int n=0)
+  {
+    if (!n) {
+      n=seed_[0]+seed_[1]+seed_[2];
+      n &= 255;
+    }
+    for (int i=0;i<n;i++)
+      randomDouble();
+  }
   //@}
   
   
@@ -1013,4 +1102,9 @@ protected:
   //@}
 };
 #endif
+#ifndef COIN_DETAIL
+#define COIN_DETAIL_PRINT(s) {}
+#else
+#define COIN_DETAIL_PRINT(s) s
+#endif
 #endif
